@@ -40,6 +40,35 @@
 using namespace std;
 using namespace ost;
 
+const char *ScriptChecks::chkKeywords(Line *line, ScriptImage *img)
+{
+	char proto[80];
+	Name *scr = img->getCurrent();
+	unsigned idx = 0;
+	const char *cp;
+
+	if(getMember(line))
+		return "no members defined";
+
+	if(hasKeywords(line))
+		return "keywords defined, not used";
+
+	if(!line->argc)
+		return "keyword list missing";
+
+	while(NULL != (cp = getOption(line, &idx)))
+	{
+		if(!isalpha(*cp) && !isdigit(*cp))
+			return "invalid keyword entry";
+	}
+
+	snprintf(proto, sizeof(proto), "keywords.%s", scr->name);
+	if(img->getPointer(proto))
+		return "keywords already defined for this function";
+	img->setPointer(proto, line);
+	return "";
+}
+
 const char *ScriptChecks::chkUse(Line *line, ScriptImage *img)
 {
 	return ScriptBinder::check(line, img);
@@ -57,6 +86,11 @@ const char *ScriptChecks::chkCall(Line *line, ScriptImage *img)
 {
 	const char *cp = getMember(line);
 	unsigned idx = 0;
+	Line *list = NULL;
+	char proto[256];
+	unsigned len = 0, diff;
+	Name *scr = img->getCurrent();
+	char *p;
 
 	if(cp)
 		return "members not used in this command";
@@ -65,6 +99,25 @@ const char *ScriptChecks::chkCall(Line *line, ScriptImage *img)
 	if(!cp)
 		return "target label missing";
 
+	if(*cp == '&')
+		++cp;
+
+	if(strchr(cp, ':') || stricmp(line->cmd, "call"))
+	{		
+		snprintf(proto, sizeof(proto), "keywords.%s", cp);
+		list = (Line *)img->getPointer(proto);
+	}
+	else if(!stricmp(line->cmd, "call"))
+	{
+		snprintf(proto, sizeof(proto), "keywords.%s", scr->name);
+		p = strchr(proto, ':');
+		if(!p)
+			p = proto + strlen(proto);
+		diff = p - proto;
+		snprintf(proto + diff, sizeof(proto) - diff, "::%s", cp);
+		list = (Line *)img->getPointer(proto);
+	}
+
 	if(*cp == '^' || *cp == '@')
 		return "invalid label used";
 
@@ -72,6 +125,22 @@ const char *ScriptChecks::chkCall(Line *line, ScriptImage *img)
 	if(cp)
 		return "only single target label allowed";
 
+	if(!list)
+		return NULL;
+
+	idx = 0;
+	while(NULL != (cp = getOption(list, &idx)))
+	{
+		snprintf(proto + len, sizeof(proto) - len, "=%s", cp);
+		len = strlen(proto);
+	}
+
+	if(len)
+	{
+		if(!useKeywords(line, proto))
+			return "invalid keyword used for function call";
+	}
+
 	return NULL;
 }
 
