@@ -403,12 +403,18 @@ Script::Name *ScriptCompiler::include(const char *token)
 {
 	char buffer[256];
 	const char *prefix = cmds->getLast("include");
+	const char *cp;
 	Name *inc;
 
 	if(!prefix)
 		return NULL;
 
-	snprintf(buffer, sizeof(buffer), "%s/%s.mac", prefix, token);
+	snprintf(buffer, sizeof(buffer), "virtual.%s", token);
+	cp = cmds->getLast(buffer);
+	if(cp)
+		snprintf(buffer, sizeof(buffer), "%s/%s_%s.mac", prefix, token, cp);
+	else
+		snprintf(buffer, sizeof(buffer), "%s/%s.mac", prefix, token);
 
 	inc = getScript(token);
 	if(!inc)
@@ -470,7 +476,7 @@ int ScriptCompiler::compile(istream *str, char *name, const char *scrname)
 	streampos pos;
 	bool ignore;
 	scrAccess access = scrPUBLIC;
-	char temp[32];
+	char temp[64];
 	char *pmode = NULL;
 	char *ftoken = NULL;
 	bool bm = false;
@@ -712,18 +718,6 @@ first:
 				continue;
 			}
 
-			// compatibility mode... ::label
-			if(!strnicmp(token, "::", 2))
-			{
-				if(bm)
-					pmode = "protected";
-				else
-					pmode = "public";
-				name = token + 2;
-				token = "::";
-				break;
-			}
-
 			if(!cmds->ripple && !embed && !apps)
   			  if(!stricmp(token, "private") || !stricmp(token,
 "protected") || !stricmp(token, "public") || !stricmp(token, "program")
@@ -767,14 +761,14 @@ repname1:
 				goto repname1;
 			}
 
-			if(!strnicmp(token, "func", 4) && !cmds->ripple && !embed && !apps)
+			if(!strnicmp(token, "func", 4) && !cmds->ripple && !embed && !apps && !strchr(token, ':'))
 			{
 				ftoken = "fconst";
 				pmode = "function";
 				goto repname1;
 			}
 
-			if(!strnicmp(token, "proc", 4) && !cmds->ripple && !embed && !apps)
+			if(!strnicmp(token, "proc", 4) && !cmds->ripple && !embed && !apps && !strchr(token, ':'))
 			{
 				pmode = "function";
 				goto repname1;
@@ -799,12 +793,6 @@ repname1:
 				break;
 			}
 
-			if(!strncmp(token, "::", 2))
-			{
-				name = token + 2;
-				break;
-			}
-			
 			if((cmds->ripple || apps) && *token == '[')
 			{
 				name = token + 1;
@@ -816,7 +804,7 @@ repname1:
 			}
 
 			tlen = (int)strlen(token);
-			if(token[tlen - 1] == ':')
+			if(token[tlen - 1] == ':' && !cmds->ripple && !apps && !embed)
 			{
 				token[tlen - 1] = 0;
 				name = token;
@@ -824,6 +812,7 @@ repname1:
 				break;
 			}
 
+
 			if(*token == '^')
 			{
 				if(!trapflag)
@@ -908,6 +897,25 @@ noexec:
 			snprintf(catchname, sizeof(catchname), "use.%s", token);
 			token = catchname;
 		}
+		else if(!stricmp(token, "virtual") && !embed && !cmds->ripple && scrStream == (istream *)&scrSource)
+		{
+			token = getToken();
+			if(!token)
+				continue;
+
+			if(strchr(token, '/'))
+				continue;
+
+			if(inccount > 255)
+				continue;
+
+			snprintf(temp, sizeof(temp), "virtual.%s", token);
+			if(!cmds->getLast(temp))
+				cmds->setValue(temp, "none");
+
+			incfiles[inccount++] = alloc(token);
+			continue;
+		}
 		else if((!stricmp(token, "requires") || !stricmp(token, "import")) && scrStream == (istream *)&scrSource && !cmds->ripple && !apps && !embed)
 		{
 			token = getToken();
@@ -934,16 +942,21 @@ noexec:
 			pretoken = token;
 			token = "expr";
 		}
-		else if(*token == '&' && !cmds->ripple)
+		else if(*token == '&' && !cmds->ripple && !apps)
 		{
 			pretoken = token;
 			token = "call";
 		}
-		else if(!strnicmp(token, "*::", 3) && !cmds->ripple)
+		else if(!strnicmp(token, "*::", 3) && !cmds->ripple && !apps)
 		{
 			pretoken = token + 3;
 			token = "call";
 		}
+		else if(!strnicmp(token, "::", 2) && !cmds->ripple && !apps)
+		{
+			pretoken = token + 2;
+			token = "call";
+		}
 		else if(strstr(token, "::") && !cmds->ripple && !apps)
 		{
 			pretoken = token;
