@@ -981,6 +981,9 @@ bool ScriptInterp::redirect(bool evflag)
 		return true;
 	}
 
+	if(*label == '&')
+		++label;
+
 	if(strstr(label, "::"))
 		shortflag = false;
 
